!
!  walk.f90:        Random walk simulation 
!								 
!  taken from: "Projects in Computational Physics" by Landau and Paez
!	       copyrighted by John Wiley and Sons, New York          
!                                                                  
!  written by: students in PH465/565, Computational Physics,      
!	       at Oregon State University                         
!              code copyrighted by RH Landau                    
!  supported by: US National Science Foundation, Northwest Alliance
!                for Computational Science and Engineering (NACSE),
!                US Department of Energy 	
!
!  modified for SUN Studio12 f95 compiler: Ch. Elster, July 2008                       
!								       
!          Data is saved as sqrt(steps), distance        
!----------------------------------------------------------------------

      Program walk

      implicit none

      integer :: np,ntrial
      parameter (np=100000,ntrial=100)

      integer :: i, it, j, max, iflag
      double precision :: drand, root2, theta, x, y, r(np)
      double precision :: xi(np), yi(np)

      integer :: kread, kwrite, kpunch

      data kread/5/, kwrite/6/, kpunch/20/


!        set parameters (# of steps)

      max   = 10000
      iflag = 1
      root2 = sqrt(2.d0)

!        open file, seed generator

      open (kwrite, FILE='walk.dat', Status='Unknown')
      open (kpunch, FILE='walk2.dat', Status='Unknown')

!         plant seed

      r(1)=drand(iflag)
      iflag=0 

!         clear array
      do  j=1, max
      r(j) = 0.d0
      end do

!         average over 100 trials
      do  j = 1, ntrial
          x = 0.d0
          y = 0.d0

!         take max steps
        do  i = 1, max
            x = x + (drand(iflag)-0.5d0)*2.0*root2
            y = y + (drand(iflag)-0.5d0)*2.0*root2
            r(i) = r(i)+ sqrt(x*x + y*y)
            xi(i)=x
            yi(i)=y
        end do
       end do

      

!        output data for plot of r vs. sqrt(N) and the actual walk

       do i = 1, max
          write (kwrite,*) sqrt(dble(i)), ' ', r(i)*0.01d0
          write (kpunch,*) xi(i)*0.01d0,' ',yi(i)*0.01d0
       end do
       close(kwrite)

       stop 'data saved in walk.dat and walk2.dat'
       end
